// UI.CPP
// Basic user interface for a cdrLabel plug-in

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "resource.h"
#include "plugin.h"

static HINSTANCE hInstance = NULL; // Our DLL instance
static MINIGRAPHIC *gfx = NULL;    // The MINIGRAPHIC we've been given
static POINT position = {0, 0};    // Where our dialog should be placed

// The actual computation engine.  Pass in preview to make it go faster
// but less accurate (i.e. only process 1/4 pixels or use integer instead
// of FP).
extern void ImageIt(int preview, int brightness, int contrast,
		    MINIGRAPHIC *src, MINIGRAPHIC *dst);

// Windows standard DLLMain, nothing to do except get hInstance
BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  hInstance = (HINSTANCE)hModule;
  
  return TRUE;
}

// CenterDialog will place a dialog in the center of either its parent window
// or the screen if there is no parent
static void CenterDialog( HWND hwndDlg )
{
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

  hwndOwner = GetParent(hwndDlg);
  if (hwndOwner == NULL) hwndOwner = GetDesktopWindow(); 
  GetWindowRect(hwndOwner, &rcOwner); 
  GetWindowRect(hwndDlg, &rcDlg); 
  CopyRect(&rc, &rcOwner); 
  OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
  OffsetRect(&rc, -rc.left, -rc.top); 
  OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 
  SetWindowPos(hwndDlg, HWND_TOP, rcOwner.left + (rc.right / 2),
	       rcOwner.top + (rc.bottom / 2), 0, 0, SWP_NOSIZE); 
}

// PositionWindow moves a window to the specifiec point while checking to
// make sure it is still visible.  If the POINT == {0,0} then will center
static void PositionWindow( HWND hdlg, POINT *pt )
{
  HDC hdc;
  if (pt->x!=0 && pt->y!=0) {
    hdc=CreateIC("DISPLAY", "", "", NULL);
    pt->x = min( GetDeviceCaps(hdc, HORZRES)-100, pt->x);
    pt->x = max( pt->x, 0 );
    pt->y = min( GetDeviceCaps(hdc, VERTRES)-100, pt->y);
    pt->y = max( pt->y, 0 );
    DeleteDC(hdc);
    SetWindowPos(hdlg, HWND_TOP, pt->x, pt->y, 0,0, SWP_NOSIZE);
  } else
    CenterDialog(hdlg);
}

// Stores the dialog position in the POINT* passed in
static void SaveWindowPosition( HWND hdlg, POINT *pt )
{
  RECT saverect;
  GetWindowRect(hdlg, &saverect);
  pt->x = saverect.left;
  pt->y = saverect.top;
}



//---------------------------------------------------------------------------
// GraphicFilterInitialize takes care of any initialization and version
// checking, and returns TRUE if it is safe to continue.
EXPORT int GraphicFilterInitialize( MINIGRAPHICPARAM *param )
{
  DWORD type, len;

  // First, check that we can grok this plugin version!
  if (!param) return FALSE;
  if (param->version != PLUGINVERSION) return FALSE;

  // Safe to continue
  InitCommonControls(); // We use a slider bar...
  strncpy(param->title, "Brightness and Contrast", MAXTITLE);
  param->user = NULL;  // No preferences for this plugin

  // Try and load the last saved dialog position from the registry
  type = REG_BINARY;
  len = sizeof(POINT);
  RegQueryValueEx(param->userKey, "Position", 0, &type,
		  (unsigned char *)&position, &len);
  
  return TRUE;
}


// Dialog is a standard DialogProc.  Its LPARAM is a pointer to the param
// structure passed in from cdrLabel.  On an update we do a preview, and
// on OK or Cancel we remove any temporary MINIGRAPHICS and return the
// final processed output.
static BOOL CALLBACK Dialog(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static MINIGRAPHICPARAM *param;
  static MINIGRAPHIC *mg;

  int contrast, brightness;

  switch(msg) {

  case WM_HSCROLL: // Sent by the thumbscrolls
    // Get the current values of the scrolls and display them
    contrast = SendDlgItemMessage(hDlg, IDC_CONTRAST, TBM_GETPOS, 0, 0);
    SetDlgItemInt(hDlg, IDC_CONT, contrast, FALSE);
    brightness = SendDlgItemMessage(hDlg, IDC_BRIGHTNESS, TBM_GETPOS, 0, 0);
    SetDlgItemInt(hDlg, IDC_BRITE, brightness, FALSE);
    // Now perform the processing
    ImageIt( FALSE, brightness, contrast, gfx, mg );
    // Display it!
    param->PreviewMiniGraphic( mg );
    return TRUE;

  case WM_INITDIALOG: // Sent by Windoze
    param = (MINIGRAPHICPARAM*)lParam; // The LPARAM is a MINIGRAPHICPARAM
    PositionWindow(hDlg, &position); // Move dialog to last spot
    
    // MG will be our modified graphic
    mg = param->CreateMiniGraphic( gfx->width, gfx->height );

    // Setup contrast scroller
    SendDlgItemMessage(hDlg, IDC_CONTRAST, TBM_SETRANGE, TRUE,
		       (LPARAM)MAKELONG(0, 100));
    SendDlgItemMessage(hDlg, IDC_CONTRAST, TBM_SETPAGESIZE, 0, (LPARAM)10);
    SendDlgItemMessage(hDlg, IDC_CONTRAST, TBM_SETPOS, TRUE, (LPARAM) 50);

    // Same for brightness
    SendDlgItemMessage(hDlg, IDC_BRIGHTNESS, TBM_SETRANGE, TRUE,
		       (LPARAM)MAKELONG(0, 100));
    SendDlgItemMessage(hDlg, IDC_BRIGHTNESS, TBM_SETPAGESIZE, 0, (LPARAM)10);
    SendDlgItemMessage(hDlg, IDC_BRIGHTNESS, TBM_SETPOS, TRUE, (LPARAM) 50);

    // Focus on the 1st item in the dialog
    SetFocus(GetDlgItem(hDlg, IDC_CONTRAST));

	// Preview...
	SendMessage(hDlg, WM_HSCROLL, 0, 0);
    return FALSE;


  case WM_COMMAND: // Sent by a button
    switch (LOWORD(wParam)) {
    case IDOK:
      SaveWindowPosition(hDlg, &position); // Store window position
      param->FreeMiniGraphic( gfx ); // Get rid of ORIGINAL graphic
      gfx = mg; // We'll return the new graphic
      EndDialog(hDlg, TRUE);
      return TRUE;
		
    case IDCANCEL:
      SaveWindowPosition(hDlg, &position); // Save window position
      param->FreeMiniGraphic( mg ); // Get rid of TEMP graphic
      EndDialog(hDlg, FALSE);
      return TRUE;
		
    }
    break;
  }
  return FALSE;
}


// GraphicFilterUserInterface is called by cdrLabel whenever our menu item
// is selected.  We receive a graphic to work on and return the (possibly)
// modified result of our operation.
EXPORT MINIGRAPHIC *GraphicFilterUserInterface( MINIGRAPHICPARAM *param,
						HWND parent,
						MINIGRAPHIC *graphic )
{
  gfx = graphic; // May be changed by the DialogProc
  // Display the dialog modally
  DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG), parent, Dialog,
		 (LPARAM)param);
  // Return either new or old graphic to cdrLabel
  return gfx;
}


// GraphicFilterUninitialize is called when cdrLabel is shutting down
// We'd free any memory we had previously allocated and store any permanent
// options like dialog positions or dialog settings
EXPORT void GraphicFilterUninitialize( MINIGRAPHICPARAM *param )
{
  // Save where the dialog is, makes for a nicer UI
  RegSetValueEx(param->userKey, "Position", 0, REG_BINARY,
		(unsigned char*)&position, sizeof(POINT));
}

